package jp.co.sds_corp.util.audio;

import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Control;
import javax.sound.sampled.Line;
import javax.sound.sampled.Mixer;

/**
* Audioデバイスの制御を行います。
* @author	小見山　聡
*/
public class VolumeControl
{
	public VolumeControl() {
	}

	/**
	* 全てのAudioデバイスの情報を表示します。
	*/
	public void printTree() throws Exception {
		Mixer.Info[] minf = AudioSystem.getMixerInfo();
		System.out.println( "Mixer:" + minf.length );
		for ( int i = 0; i < minf.length; i++ ) {
			System.out.println( "" + i + ":" + minf[i] );
			printMixerInfo( minf[i] );
		}
	}

	/**
	* Mixerの情報を表示します。
	* @param	minf	Mixer情報
	*/
	public void printMixerInfo( Mixer.Info minf ) throws Exception {
		Mixer mix = AudioSystem.getMixer( minf );
		printLine( mix );

		Line.Info[] linf;
		linf = mix.getSourceLineInfo();
		System.out.println( " Source Line:" + linf.length );
		for ( int j = 0; j < linf.length; j++ ) {
			printLineInfo( linf[j] );
		}
		linf = mix.getTargetLineInfo();
		System.out.println( " Target Line:" + linf.length );
		for ( int j = 0; j < linf.length; j++ ) {
			printLineInfo( linf[j] );
		}
	}

	/**
	* Lineの情報を表示します。
	* @param	linf	Line情報
	*/
	public void printLineInfo( Line.Info linf ) throws Exception {
		Line l = AudioSystem.getLine( linf );
		printLine( l );
	}

	/**
	* Lineの情報を表示します。
	* @param	l		Line情報
	*/
	public void printLine( Line l ) throws Exception {
		System.out.println( " |-" + l.getLineInfo() );
		Control[] ctr = l.getControls();
		for ( int k = 0; k < ctr.length; k++ ) {
			printControl( ctr[k] );
		}
	}

	/**
	* Controlの情報を表示します。
	* @param	ctr		Control情報
	*/
	public void printControl( Control ctr ) {
		System.out.println( "   |-" + ctr );
	}

	/**
	* ボリュームの値をdB(デシベル)に変換します。
	* @param	val		ボリューム値
	* @return	dB値
	*/
	float getdB( float val ) {
		return (float )(Math.log(val==0.0?0.0001:val)/Math.log(10.0)*20.0);
	}

	/**
	* ボリューム制御デバイスを検索します。
	*/
	public void findGainControl() throws Exception {
		Mixer.Info[] minf = AudioSystem.getMixerInfo();
		Control ctr = null;
		for ( int i = 0; i < minf.length; i++ ) {
			Mixer mix = AudioSystem.getMixer( minf[i] );
			printLine( mix );
/*
			ctr = mix.getControl( FloatControl.Type.MASTER_GAIN );
			if ( ctr != null ) {
				break;
			}
*/
		}
		if ( ctr == null ) {
			System.out.println( "not found." );
			return;
		}
		System.out.println( ctr );
	}

	public static void main( String[] args ) {
		try {
			VolumeControl vol = new VolumeControl();
			vol.printTree();

			System.out.println( "\nfind" );
			vol.findGainControl();
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}

		System.exit( 0 );
	}
}
