package jp.co.sds_corp.util.charset;

import java.lang.reflect.Method;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CoderResult;

public class CharsetDecoderWrapper extends CharsetDecoder {
	CharsetDecoder decoder;
	Method decodeLoop;

	public CharsetDecoderWrapper(CharsetDecoder org) {
		super(org.charset(), org.averageCharsPerByte(), org.maxCharsPerByte());
		this.decoder = org;
		try {
			decodeLoop = org.getClass().getDeclaredMethod("decodeLoop",ByteBuffer.class, CharBuffer.class);
			decodeLoop.setAccessible(true);
		} catch (Exception ex) {
			throw(new RuntimeException("get method error.", ex));
		}
	}

	protected CoderResult decodeLoop(CharsetDecoder decoder, ByteBuffer in, CharBuffer out) {
		CoderResult result;
		try {
			result = (CoderResult )decodeLoop.invoke(decoder, in, out);
		} catch (Exception ex) {
			throw(new RuntimeException("invoke error.", ex));
		}
		return result;
	}

	@Override
	protected CoderResult decodeLoop(ByteBuffer in, CharBuffer out) {
		CoderResult result = decodeLoop(decoder, in, out);
		if (result.isMalformed() || result.isUnmappable()) {
			int pos = in.position();
			throw(new CharsetConvertException("decode error. " + result
					+ " (" + pos + ":" + result.length() + ")"));
		}
		return result;
	}

}
