package	jp.co.sds_corp.util.common;

import java.io.File;
import java.io.IOException;
import java.net.JarURLConnection;
import java.net.URL;
import java.util.jar.Attributes;
import java.util.jar.JarFile;

/**
* Jarファイルの情報を取得します。
*/
public class CmnManifest
{
	JarFile jf;

	/**
	* 指定された Class が格納されているJarファイルで初期化します
	* @param	cls		検索するClass
	*/
	public CmnManifest( Class<?> cls ) throws Exception {
		String fn = CmnString.replaceAll( cls.getName(), ".", "/" );
		fn = fn + ".class";
		URL url = getClass().getClassLoader().getResource( fn );
		jf = ((JarURLConnection )url.openConnection()).getJarFile();
	}

	/**
	* 指定されたファイルで初期化します
	* @param	file	Jarファイル名
	*/
	public CmnManifest( File file ) throws Exception {
		jf = new JarFile( file );
	}

	/**
	* タイトル名を取得します。
	* Implementation-Title 属性を取得します。
	*/
	public String getTitle() throws IOException {
		Attributes atr = jf.getManifest().getMainAttributes();
		return (String )atr.getValue( "Implementation-Title" );
	}

	/**
	* バージョンを取得します。
	* Implementation-Version 属性を取得します。
	*/
	public String getVersion() throws IOException {
		Attributes atr = jf.getManifest().getMainAttributes();
		return (String )atr.getValue( "Implementation-Version" );
	}

	/**
	* ベンダ名を取得します。
	* Implementation-Vendor 属性を取得します。
	*/
	public String getVendor() throws IOException {
		Attributes atr = jf.getManifest().getMainAttributes();
		return (String )atr.getValue( "Implementation-Vendor" );
	}

	/**
	* 指定されたエントリ名の Attributes を返します。
	* @param	key		エントリ名
	*/
	public Attributes getAttributes( String key ) throws IOException {
		return jf.getManifest().getAttributes( key );
	}

	/**
	* 指定されたエントリ名からタイトル名を取得します。
	* Implementation-Title 属性を取得します。
	* @param	key		エントリ名
	*/
	public String getTitle( String key ) throws IOException {
		Attributes atr = getAttributes( key );
		if (atr == null) {
			return null;
		}
		return (String )atr.getValue( "Implementation-Title" );
	}

	/**
	* 指定されたエントリ名からバージョンを取得します。
	* Implementation-Version 属性を取得します。
	* @param	key		エントリ名
	*/
	public String getVersion( String key ) throws IOException {
		Attributes atr = getAttributes( key );
		if (atr == null) {
			return null;
		}
		return (String )atr.getValue( "Implementation-Version" );
	}

	/**
	* 指定されたエントリ名からベンダ名を取得します。
	* Implementation-Vendor 属性を取得します。
	* @param	key		エントリ名
	*/
	public String getVendor( String key ) throws IOException {
		Attributes atr = getAttributes( key );
		if (atr == null) {
			return null;
		}
		return (String )atr.getValue( "Implementation-Vendor" );
	}

}
