package	jp.co.sds_corp.util.common;

import java.util.StringTokenizer;

/**
* 文字列操作を補助します。
* @author	杉田　健児
*/
public class CmnString {

	/**
	* 文字列をデリミタで分割します。
	* デリミタが連続している場合、間のカラムは""（長さ0の文字列）として
	* 返されます。
	* @param	data		分割する文字列
	* @param	delim		デリミタ
	*/
	public static String[] getTokens( String data, String delim ) {
		String[] arr = getStringToken( data, delim, true );
		int nDlm = 0;
		for ( int i = 0; i < arr.length; i++ ) {
			if ( arr[i].equals( delim )) {
				nDlm++;
			}
		}
		String[] ret = new String[nDlm+1];
		int nPos = 0;
		for ( int i = 0; i < arr.length; i++ ) {
			if ( arr[i].equals( delim )) {
				if ( ret[nPos] == null ) {
					ret[nPos] = "";
				}
				nPos++;
			} else {
				ret[nPos] = arr[i];
			}
		}
		if ( ret[nPos] == null ) {
			ret[nPos] = "";
		}
		return ret;
	}

	/**
	* 文字列をデリミタで分割します。
	* 区切り文字はトークンを分けるだけのものと見なされます。
	* トークンは区切り文字でない文字が 1 個以上連続している部分です。
	* @param	data		分割する文字列
	* @param	delim		デリミタ
	*/
	public static String[] getStringToken( String data, String delim ) {
		return getStringToken( data, delim, false );
	}

	/**
	* 文字列をデリミタで分割します。
	* フラグが false の場合、区切り文字はトークンを分けるだけのものと
	* 見なされます。
	* トークンは区切り文字でない文字が 1 個以上連続している部分です。
	* フラグが true の場合は、区切り文字はそれ自体がトークンと見なされます。
	* トークンは、1 個の区切り文字か、区切り文字でない文字が 
	* 1 個以上連続している部分です。
	* @param	data		分割する文字列
	* @param	delim		デリミタ
	* @param	returnDelims	デリミタを返すかどうかのフラグ
	*/
	public static String[] getStringToken( String data, String delim,
							boolean returnDelims ) {
		StringTokenizer tok;
		if ( delim == null ) {
			tok = new StringTokenizer( data );
		} else {
			tok = new StringTokenizer( data, delim, returnDelims );
		}
		int nNum = tok.countTokens();
		String[] res = new String[nNum];
		for ( int i = 0; i < nNum; i++ ) {
			if ( delim == null ) {
				res[i] = tok.nextToken();
			} else {
				res[i] = tok.nextToken( delim );
			}
		}
		return res;
	}
    /**
	* 文字列中の指定された文字列を置き換えます。
	* JDK1.3対応用文字列置き換えメソッド
	* @param	orgStr		元の文字列
	* @param	oldStr		置換前文字列
	* @param	newStr		置換後文字列
	*/
	public static String replaceAll( String orgStr, String oldStr, String newStr ){
		StringBuilder str = new StringBuilder();
		int top = 0;
		int next = orgStr.indexOf( oldStr, 0 );
		while( next != -1 ){
			str.append( ( orgStr.substring( top, next ) + newStr ));
			top = next+oldStr.length();
			next = orgStr.indexOf( oldStr, top );
		}
		str.append( orgStr.substring( top ));

		return str.toString();
	}

	/**
	* 文字列をエスケープシーケンスで表現します。
	* ex. 改行文字は"\n"の2文字に変換されます。
	*/
	public static String escape( String str ) {
		StringBuffer buf = new StringBuffer();
		for ( int i = 0; i < str.length(); i++ ) {
			char ch = str.charAt( i );
			switch ( ch ) {
			case '\r': buf.append( "\\r" ); break;
			case '\n': buf.append( "\\n" ); break;
			case '\t': buf.append( "\\t" ); break;
			default: buf.append( ch );
			}
		}
		return buf.toString();
	}

	/**
	* エスケープシーケンスを変換します。
	* ex. '\\'と'n'の2文字は、改行文字に変換されます。
	*/
	public static String unescape( String str ) {
		if ( str == null ) {
			return null;
		}
		StringBuffer buf = new StringBuffer();
		boolean esc = false;
		for ( int i = 0; i < str.length(); i++ ) {
			char ch = str.charAt( i );
			if ( esc ) {
				esc = false;
				switch ( ch ) {
				case '\\':
					buf.append( '\\' );
					break;
				case 'r':
					buf.append( '\r' );
					break;
				case 'n':
					buf.append( '\n' );
					break;
				case 't':
					buf.append( '\t' );
					break;
				default:
					buf.append( '\\' );
					buf.append( ch );
					break;
				}
				continue;
			} else if ( ch == '\\' ) {
				esc = true;
				continue;
			}
			buf.append( ch );
		}
		return buf.toString();
	}

}
