package	jp.co.sds_corp.util.common;

import java.awt.Color;

public class ColorDecoder
{
	static String[] defSyms = {
		"black", "blue", "cyan", "darkGray", "gray",
		"green", "lightGray", "magenta", "orange", "pink",
		"red", "white", "yellow"
	};
	static Color[] defCols = {
		Color.black, Color.blue, Color.cyan, Color.darkGray, Color.gray,
		Color.green, Color.lightGray, Color.magenta, Color.orange, Color.pink,
		Color.red, Color.white, Color.yellow
	};

	/**
	* 文字列をColorに変換します。
	* #RRGGBB 形式、もしくはデフォルトの色(ex. red)を指定することができます。
	*/
	public static Color decode( String name ) throws ColorFormatException {
		Color ret = null;
		try {
			ret = Color.decode( name );
		} catch ( NumberFormatException nex ) {
		}
		if ( ret != null ) {
			return ret;
		}
		for ( int i = 0; i < defCols.length; i++ ) {
			if ( name.equals( defSyms[i] )) {
				return defCols[i];
			}
		}
		throw( new ColorFormatException( "For input String:" + name ));
	}

	/**
	* Colorを文字列に変換します。
	* デフォルトの色(ex. red)の場合は"red"を、それ以外は#RRGGBB形式で返します。
	*/
	public static String encode( Color col ) {
		for ( int i = 0; i < defCols.length; i++ ) {
			if ( col.equals( defCols[i] )) {
				return defSyms[i];
			}
		}
		String ret = encodeHex( col );
		return ret;
	}

	/**
	* Colorを#RRGGBB形式の文字列に変換します。
	*/
	public static String encodeHex( Color col ) {
		String ret = "000000" + Integer.toHexString( col.getRGB() & 0xffffff );
		ret = "#" + ret.substring( ret.length() - 6 );
		return ret;
	}

}
