package jp.co.sds_corp.util.common;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.io.ObjectOutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Properties;
import java.util.jar.JarEntry;
import java.util.jar.JarInputStream;

/**
* Servletからファイルを取得します。
* @author	杉田　健児
*/
public class ServletDownloader
{
	private String	servletName;

	/**
	* 指定されたServletからファイルを取得するよう初期化します。
	* @param	servletName	サーブレット名
	*/
	public ServletDownloader( String servletName ) {
		this.servletName = servletName;
	}

	/**
	* サーブレットからファイルを取得するRequestを送信します
	* @param	fileName	取得するファイル名
	*/
	public URLConnection requestServletFile( String fileName )
		throws Exception
	{
		URL url = new URL( servletName );
		URLConnection ucon = url.openConnection();
		ucon.setDoOutput( true );
		ucon.setUseCaches( false );
		ucon.setDoInput( true );
		String getName =
					new String( fileName.getBytes( "EUC-JP" ), "8859_1" );
		ObjectOutputStream oos =
					new ObjectOutputStream( ucon.getOutputStream() );
		oos.writeObject( getName );
		oos.flush();
		oos.close();

		return ucon;
	}

	/**
	* サーブレットからの応答をチェックします
	* @param	instream	サーブレットからの応答Stream
	* @return	true:正常 false:エラー
	*/
	public boolean checkStatus( InputStream instream )
		throws Exception
	{
		byte sts[] = new byte[16];
		int sts_len = 3;
		String sts_str = "";
		int len;
		int total = 0;
		for( ; total < sts_len; ) {
			len = instream.read( sts, 0, (sts_len - total) );
			total += len;
			if( len == -1 ) {
				break;
			}
			sts_str = sts_str.concat( new String( sts, 0, len ) );
		}
		if ( ( new String( sts, 0, 3 )).equals( "000" ) ) {
			return true;
		}
		return false;
	}

	/**
	* サーブレットからファイルを取得します。
	* @param	fileName	取得するファイル名
	* @return	取得したファイルの内容
	*/
	public String getTextFile( String fileName )
		throws Exception
	{
		URLConnection ucon = requestServletFile( fileName );
		InputStream instream = ucon.getInputStream();
		if ( !checkStatus( instream ) ) {
			instream.close();
			return null;
		}
		BufferedInputStream bis = new BufferedInputStream( instream );
		byte data[] = new byte[1024];
		int len;
		String hdata = "";
		for( ;; ) {
			len = bis.read( data, 0, 1024 );
			if( len == -1 ) {
				break;
			}
			hdata = hdata.concat( new String( data, 0, len ) );
		}
		bis.close();
		return hdata;
	}

	/**
	* サーブレットからJarファイルを取得し、その中からファイルを抽出します
	* @param	jarName		取得するJarファイル名
	* @param	fileName	抽出するファイル名
	* @return	抽出したファイルの内容
	*/
	public String getJarFile( String jarName, String fileName )
		throws Exception
	{
		URLConnection ucon = requestServletFile( jarName );
		InputStream instream = ucon.getInputStream();
		if( !checkStatus( instream ) ) {
			instream.close();
			return null;
		}

		JarInputStream jis = new JarInputStream( instream );

		JarEntry je;
		for( ;; ) {
			je = jis.getNextJarEntry();
			if ( je == null ) {
				break;
			}
			if ( je.getName().equals( fileName ) ) {
				break;
			}
		}
		if( je == null ) {
			return null;
		}
		BufferedInputStream bis = new BufferedInputStream( jis );
		byte data[] = new byte[1024];
		int len;
		String hdata = "";
		for( ;; ) {
			len = bis.read( data, 0, 1024 );
			if( len == -1 ) {
				break;
			}
			hdata = hdata.concat( new String( data, 0, len ) );
		}
		bis.close();
		return hdata;
	}

	/**
	* サーブレットからファイルを取得します。
	* @param	fileName	取得するファイル名
	* @return	取得したファイルの内容
	*/
	public Properties getPropertyFile( String fileName )
		throws Exception
	{
		URLConnection ucon = requestServletFile( fileName );
		InputStream instream = ucon.getInputStream();
		if( !checkStatus( instream ) ) {
			instream.close();
			return null;
		}
		BufferedInputStream bis = new BufferedInputStream( instream );

		Properties prop = new Properties();
		prop.load( bis );
		return prop;
	}

}
