package	jp.co.sds_corp.util.db;

import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.NClob;
import java.sql.ParameterMetaData;
import java.sql.PreparedStatement;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.RowId;
import java.sql.SQLException;
import java.sql.SQLXML;
import java.util.Calendar;

/**
* PreparedStatementのラッパクラスです。
* prepared SQL実行時に、SQL文と動的パラメータをログファイルに出力します。
* @author	小見山　聡
*/
public class DbPreparedStatement extends DbStatement
	implements PreparedStatement
{
	PreparedStatement stmt = null;

	public DbPreparedStatement( PreparedStatement stmt ) {
		super( stmt );
		this.stmt = stmt;
	}

	//---------------------------------------------------------------
	// PreparedStatement method.
	//---------------------------------------------------------------
	public ResultSet executeQuery() throws SQLException {
		printParameter();
		clearParameter();
		return stmt.executeQuery();
	}

	public int executeUpdate() throws SQLException {
		printParameter();
		clearParameter();
		return stmt.executeUpdate();
	}

	public void setNull( int paramIndex, int sqlType, String typeName )
		throws SQLException
	{
		addParameter( paramIndex, "null" );
		stmt.setNull( paramIndex, sqlType, typeName );
	}

	public void setBoolean( int paramIndex, boolean x )
		throws SQLException
	{
		addParameter( paramIndex, String.valueOf( x ));
		stmt.setBoolean( paramIndex, x );
	}

	public void setByte( int paramIndex, byte x ) throws SQLException {
		addParameter( paramIndex, String.valueOf( x ));
		stmt.setByte( paramIndex, x );
	}

	public void setShort( int paramIndex, short x ) throws SQLException {
		addParameter( paramIndex, String.valueOf( x ));
		stmt.setShort( paramIndex, x );
	}

	public void setInt( int paramIndex, int x ) throws SQLException {
		addParameter( paramIndex, String.valueOf( x ));
		stmt.setInt( paramIndex, x );
	}

	public void setLong( int paramIndex, long x ) throws SQLException {
		addParameter( paramIndex, String.valueOf( x ));
		stmt.setLong( paramIndex, x );
	}

	public void setFloat( int paramIndex, float x ) throws SQLException {
		addParameter( paramIndex, String.valueOf( x ));
		stmt.setFloat( paramIndex, x );
	}

	public void setDouble( int paramIndex, double x ) throws SQLException {
		addParameter( paramIndex, String.valueOf( x ));
		stmt.setDouble( paramIndex, x );
	}

	public void setBigDecimal( int paramIndex, BigDecimal x )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setBigDecimal( paramIndex, x );
	}

	public void setString( int paramIndex, String x ) throws SQLException {
		addParameter( paramIndex, x );
		stmt.setString( paramIndex, x );
	}

	public void setBytes( int paramIndex, byte x[] ) throws SQLException {
		addParameter( paramIndex, new String( x ));
		stmt.setBytes( paramIndex, x );
	}

	public void setDate( int paramIndex, java.sql.Date x )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setDate( paramIndex, x );
	}

	public void setTime( int paramIndex, java.sql.Time x )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setTime( paramIndex, x );
	}

	public void setTimestamp( int paramIndex, java.sql.Timestamp x )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setTimestamp( paramIndex, x );
	}

	public void setAsciiStream( int paramIndex, InputStream x, int length )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setAsciiStream( paramIndex, x, length );
	}

	/**
	* @deprecated
	*/
	public void setUnicodeStream( int paramIndex, InputStream x, int length )
		throws SQLException
	{
// deprecated.
//		addParameter( paramIndex, x == null ? "null" : x.toString() );
//		stmt.setUnicodeStream( paramIndex, x, length );
	}

	public void setBinaryStream( int paramIndex, InputStream x, int length )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setBinaryStream( paramIndex, x, length );
	}

	public void clearParameters() throws SQLException {
		clearParameter();
		stmt.clearParameters();
	}

	public void setObject( int paramIndex, Object x, int targetSqlType )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setObject( paramIndex, x, targetSqlType );
	}

	public void setObject( int paramIndex, Object x )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setObject( paramIndex, x );
	}

	public boolean execute() throws SQLException {
		printParameter();
		clearParameter();
		return stmt.execute();
	}

	//--------------------------JDBC 2.0-----------------------------
	public void addBatch() throws SQLException {
		stmt.addBatch();
	}

	public void setCharacterStream( int paramIndex, Reader reader, int length )
		throws SQLException
	{
		addParameter( paramIndex, reader == null ? "null" : reader.toString() );
		stmt.setCharacterStream( paramIndex, reader, length );
	}

	public void setRef( int i, Ref x ) throws SQLException {
		addParameter( i, x == null ? "null" : x.toString() );
		stmt.setRef( i, x );
	}

	public void setBlob( int i, Blob x ) throws SQLException {
		addParameter( i, x == null ? "null" : x.toString() );
		stmt.setBlob( i, x );
	}

	public void setClob( int i, Clob x ) throws SQLException {
		addParameter( i, x == null ? "null" : x.toString() );
		stmt.setClob( i, x );
	}

	public void setArray( int i, Array x ) throws SQLException {
		addParameter( i, x == null ? "null" : x.toString() );
		stmt.setArray( i, x );
	}

	public ResultSetMetaData getMetaData() throws SQLException {
		return stmt.getMetaData();
	}

	public void setDate( int paramIndex, java.sql.Date x, Calendar cal )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setDate( paramIndex, x );
	}

	public void setTime( int paramIndex, java.sql.Time x, Calendar cal )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setTime( paramIndex, x, cal );
	}

	public void setTimestamp( int paramIndex, java.sql.Timestamp x, Calendar cal )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setTimestamp( paramIndex, x, cal );
	}

	public void setNull( int paramIndex, int sqlType ) throws SQLException {
		addParameter( paramIndex, "null" );
		stmt.setNull( paramIndex, sqlType );
	}

	//------------------------- JDBC 3.0 -----------------------------------
	public void setURL( int i, java.net.URL url ) throws SQLException { 
		addParameter( i, url == null ? "null" : url.toString() );
		stmt.setURL( i, url );
	}

	public ParameterMetaData getParameterMetaData() throws SQLException { 
		return stmt.getParameterMetaData();
	}

	//------------------------- JDBC 4.0 -----------------------------------
	public void setRowId( int i, RowId x ) throws SQLException {
		addParameter( i, x == null ? "null" : x.toString() );
		stmt.setRowId( i, x );
	}

	public void setNString( int i, String x ) throws SQLException {
		addParameter( i, x );
		stmt.setNString( i, x );
	}

	public void setNCharacterStream( int i, Reader reader, long length )
		throws SQLException
	{
		addParameter( i, reader == null ? "null" : reader.toString() );
		stmt.setNCharacterStream( i, reader, length );
	}

	public void setNClob( int i, NClob x ) throws SQLException {
		addParameter( i, x == null ? "null" : x.toString() );
		stmt.setNClob( i, x );
	}

	public void setClob( int i, Reader reader, long length )
		throws SQLException
	{
		addParameter( i, reader == null ? "null" : reader.toString() );
		stmt.setClob( i, reader, length );
	}

	public void setBlob( int paramIndex, InputStream x, long length )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setBlob( paramIndex, x, length );
	}

	public void setNClob( int i, Reader reader, long length )
		throws SQLException
	{
		addParameter( i, reader == null ? "null" : reader.toString() );
		stmt.setNClob( i, reader, length );
	}

	public void setSQLXML( int i, SQLXML x ) throws SQLException {
		addParameter( i, x == null ? "null" : x.toString() );
		stmt.setSQLXML( i, x );
	}

	public void setObject( int paramIndex, Object x, int targetSqlType, int scale )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setObject( paramIndex, x, targetSqlType, scale );
	}

	public void setAsciiStream( int paramIndex, InputStream x, long length )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setAsciiStream( paramIndex, x, length );
	}

	public void setBinaryStream( int paramIndex, InputStream x, long length )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setBinaryStream( paramIndex, x, length );
	}

	public void setCharacterStream( int paramIndex, Reader reader, long length )
		throws SQLException
	{
		addParameter( paramIndex, reader == null ? "null" : reader.toString() );
		stmt.setCharacterStream( paramIndex, reader, length );
	}

	public void setAsciiStream( int paramIndex, InputStream x )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setAsciiStream( paramIndex, x );
	}

	public void setBinaryStream( int paramIndex, InputStream x )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setBinaryStream( paramIndex, x );
	}

	public void setCharacterStream( int i, Reader reader )
		throws SQLException
	{
		addParameter( i, reader == null ? "null" : reader.toString() );
		stmt.setCharacterStream( i, reader );
	}

	public void setNCharacterStream( int i, Reader reader )
		throws SQLException
	{
		addParameter( i, reader == null ? "null" : reader.toString() );
		stmt.setNCharacterStream( i, reader );
	}

	public void setClob( int i, Reader reader ) throws SQLException {
		addParameter( i, reader == null ? "null" : reader.toString() );
		stmt.setClob( i, reader );
	}

	public void setBlob( int paramIndex, InputStream x )
		throws SQLException
	{
		addParameter( paramIndex, x == null ? "null" : x.toString() );
		stmt.setBlob( paramIndex, x );
	}

	public void setNClob( int i, Reader reader ) throws SQLException {
		addParameter( i, reader == null ? "null" : reader.toString() );
		stmt.setNClob( i, reader );
	}

}

