package	jp.co.sds_corp.util.log;

import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;

/**
* コンソールにログを出力します。
* @author	小見山　聡
* @version	$Id$
*/
public class ConsoleLogger implements CmnLogger
{
	private static ConsoleLogger instance = new ConsoleLogger();
	private PrintStream out;
	private Level level;

	private ConsoleLogger() {
		out = System.out;
		level = Level.FINE;

		String var;
		var = System.getProperty( getClass().getName() + ".LEVEL" );
		if ( var != null ) {
			try {
				level = Level.parse( var );
			} catch ( Exception ex ) {
				out.println( "unknown level[" + var + "]." );
			}
		}
		// out.println( getClass().getName() + " level=" + level + "." );
	}

	/**
	* このクラスの唯一のインスタンスを返します。
	* @return	インスタンス
	*/
	public static ConsoleLogger getInstance() {
		return instance;
	}

	public void setLevel( Level level ) {
		this.level = level;
	}

	public Level getLevel() {
		return this.level;
	}

	public void setOut(PrintStream out) {
		this.out = out;
	}

	private String getCategory( Object obj ) {
		String ret = null;
		if ( obj instanceof Class ) {
			ret = ((Class<?> )obj).getSimpleName();
		} else if ( obj instanceof String ) {
			ret = (String )obj;
		} else {
			ret = obj.getClass().getSimpleName();
		}

		return ret;
	}

	public void print( String var ) {
		out.print( var );
	}

	public void println( String var ) {
		out.println( var );
	}

	public PrintStream printf( String format, Object... args ) {
		return out.printf( format, args );
	}

	@Override
	public boolean isLoggable(Object obj, Level level) {
		if (level.intValue() < this.level.intValue()) {
			return false;
		}
		return true;
	}

	@Override
	public boolean isTraceEnable(Object obj) {
		// TODO 自動生成されたメソッド・スタブ
		return false;
	}

	@Override
	public boolean isDebugEnable(Object obj) {
		// TODO 自動生成されたメソッド・スタブ
		return false;
	}

	@Override
	public boolean isDetailEnable(Object obj) {
		// TODO 自動生成されたメソッド・スタブ
		return false;
	}

	/**
	* ログを出力します。<br>
	* @param	level	出力するレベル
	* @param	logMsg	出力するメッセージ
	*/
	public synchronized void log( Object obj, Level level, String logMsg ) {
		if ( !isLoggable( obj, level )) {
			return;
		}
		SimpleDateFormat sdf = new SimpleDateFormat( "yyyy/MM/dd HH:mm:ss.SSS" );
		long id = Thread.currentThread().getId();
		out.println( sdf.format( new Date() )
			+ " " + String.format( "%08X", id )
			+ " [" + level + " " + getCategory(obj) + "] "
			+ logMsg );
	}

	/**
	* ログを出力します。<br>
	* @param	level	出力するレベル
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public synchronized void log( Object obj, Level level, String logMsg,
			Throwable t )
	{
		if ( !isLoggable( obj, level )) {
			return;
		}
		String error = "null:null";
		if ( t != null ) {
			error = t.getClass().getName() + ":" + t.getMessage();
		}
		log( obj, level, logMsg + ":" + error );
		exception( obj, t );
	}

	/**
	* ログを出力します。<br>
	* @param	logMsg	出力するメッセージ
	*/
	public synchronized void log( Object obj, String logMsg ) {
		log( obj, Level.FINER, logMsg );
	}

	/**
	* ログを出力します。<br>
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	public synchronized void log( Object obj, String logMsg, Throwable t ) {
		log( obj, Level.FINER, logMsg, t );
	}

	@Override
	public void trace(Object obj, String logMsg) {
		log( obj, Level.FINEST, logMsg );
	}

	@Override
	public void trace(Object obj, String logMsg, Throwable t) {
		log( obj, Level.FINEST, logMsg, t );
	}

	/**
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	@Override
	public void debug( Object obj, String logMsg ) {
		log( obj, Level.FINER, logMsg );
	}
	/**
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	@Override
	public void debug( Object obj, String logMsg, Throwable t ) {
		log( obj, Level.FINER, logMsg, t );
	}

	@Override
	public void detail(Object obj, String logMsg) {
		log( obj, Level.FINE, logMsg );
	}

	@Override
	public void detail(Object obj, String logMsg, Throwable t) {
		log( obj, Level.FINE, logMsg, t );
	}

	/**
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	@Override
	public void info( Object obj, String logMsg ) {
		log( obj, Level.INFO, logMsg );
	}
	/**
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	@Override
	public void info( Object obj, String logMsg, Throwable t ) {
		log( obj, Level.INFO, logMsg, t );
	}

	@Override
	public void warn(Object obj, String logMsg) {
		log( obj, Level.WARNING, logMsg );
	}

	@Override
	public void warn(Object obj, String logMsg, Throwable t) {
		log( obj, Level.WARNING, logMsg, t );
	}

	/**
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	*/
	@Override
	public void error( Object obj, String logMsg ) {
		log( obj, Level.SEVERE, logMsg );
	}
	/**
	* @param	obj		log の出力を行なったClass。
	* @param	logMsg	出力するメッセージ
	* @param	t		出力する例外
	*/
	@Override
	public void error( Object obj, String logMsg, Throwable t ) {
		log( obj, Level.SEVERE, logMsg, t );
	}

	@Override
	public void fatal(Object obj, String logMsg) {
		log( obj, Level.SEVERE, logMsg );
	}

	@Override
	public void fatal(Object obj, String logMsg, Throwable t) {
		log( obj, Level.SEVERE, logMsg, t );
	}

	/**
	* exceptionログを出力します。
	* @param	obj		log の出力を行なったClass。
	* @param	ex		出力する例外
	*/
	@Override
	public synchronized void exception( Object obj, Throwable ex ) {
		exception(obj, Level.INFO, ex);
	}

	@Override
	public void exception(Object obj, Level level, Throwable ex) {
		if ( ex == null ) {
			out.println( "no exception." );
			return;
		}
		for ( ;; ) {
			StackTraceElement[] stacks = ex.getStackTrace();
			for ( StackTraceElement stack : stacks ) {
				out.println( "\t" + stack.toString() );
			}
			ex = ex.getCause();
			if ( ex == null ) {
				break;
			}
			out.println( "Caused by: " + ex.toString() );
		}
	}

	@Override
	public void sql(Object obj, String sql) {
		log(obj, Level.FINE, sql);
	}

}
