package	jp.co.sds_corp.util.log;

import java.util.logging.Handler;
import java.util.logging.LogManager;
import java.util.logging.LogRecord;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

public class Log4jHandler extends Handler
{
	boolean	bDesc = false;

	public Log4jHandler() {
		Logger logger = Logger.getRootLogger();
		logger.debug( "start Log4jHandler->log4j" );
		LogManager manager = LogManager.getLogManager();
		String var;
		var = manager.getProperty( "jp.co.sds_corp.util.log.Log4jHandler.desc" );
		if ( var != null && var.equals( "true" )) {
			bDesc = true;
		}
	}

	/**
	* 与えられたログレコードをロギングします。
	* 
	* @param	record	ログレコード。
	*/
	public void publish( LogRecord record ) {
		String message = record.getMessage();
		Level level = null;

		if ( record.getLevel().equals( java.util.logging.Level.FINEST ) ||
			record.getLevel().equals( java.util.logging.Level.FINER ))
		{
			// FINEST と FINER は trace にマップ。
			level = Level.TRACE;
		} else
		if ( record.getLevel().equals( java.util.logging.Level.FINE )) {
			// FINE は debug にマップ。
			level = Level.DEBUG;
		} else
		if ( record.getLevel().equals( java.util.logging.Level.CONFIG )) {
			// CONFIG は infoにマップ。
			level = Level.INFO;
			message = "config: " + message;
		} else
		if ( record.getLevel().equals( java.util.logging.Level.INFO )) {
			level = Level.INFO;
		} else
		if ( record.getLevel().equals( java.util.logging.Level.WARNING )) {
			level = Level.WARN;
		} else
		if ( record.getLevel().equals( java.util.logging.Level.SEVERE )) {
			level = Level.ERROR;
			if (record.getMessage().length() >= 5) {
				String head = record.getMessage().substring(0, 5);
				if ( "fatal".equals(head.toLowerCase())) {
					level = Level.FATAL;
				}
			}
		} else {
			// しょうがないのでinfoにマップ。
			level = Level.INFO;
			message = "[" + record.getLevel().intValue() + "]"
				+ record.getMessage();
		}
		if ( bDesc ) {
			message = "(" + record.getSourceClassName()
				+ "#" + record.getSourceMethodName() + ") " + message;
		}

		Logger log4jLogger = Logger.getLogger( record.getLoggerName() );
		if ( record.getThrown() == null ) {
			log4jLogger.log( level, message );
		} else {
			log4jLogger.log( level, message, record.getThrown());
		}
	}

	/**
	* ロガーをフラッシュします。
	*/
	public void flush() {
	}

	/**
	* ロガーをクローズします。
	* 
	* @throws SecurityException	セキュリティ例外が発生した場合。
	*/
	public void close() throws SecurityException {
	}

}
