package	jp.co.sds_corp.util.math;


/**
* 関数f(x)の解をニュートン法によって求める。
*/
public class Newton
{
	/**
	* 関数f(x)
	*/
	public interface Function {
		public double of(double x);
	}

	private static double MAX;					// 最小
	private static double MIN;					// 最大
	private static double EPS = 1.0e-9;			// 収束判定
	private static int MAXITER=1000;			// 反復回数

	public static void setEps(double eps) {
		EPS = eps;
	}
	public static void setMaxIter(int maxiter) {
		MAXITER = maxiter;
	}
	public static void setMax(double max) {
		MAX = max;
	}
	public static void setMin(double min) {
		MIN = min;
	}

	/**
	* 関数f(x)の解をニュートン法によって求める。
	*/
	public static double solve(Function f, double x) {
		double fx = f.of(x);
		int c = 0;
		while (fx != 0) {
			if (c > MAXITER) {
				break;
			}
			double df = f.of(x + fx) - fx, h = fx * fx / df;
			double xPrev = x, fxPrev = fx;
			int i = 0;
			do {
				x = xPrev - h;
				fx = f.of(x);
				h /= 2;
				i++;
//				System.out.println("(" + i + ") x = " + x + "   fx(x) = " + fx);
			} while (Math.abs(fx) > Math.abs(fxPrev));
			if (Math.abs(x - xPrev) <= EPS) {
				break;
			}
			c++;
		}
		return x;
	}
}
