package	jp.co.sds_corp.util.net;

import java.net.InetAddress;
import java.util.List;
import java.util.Vector;

/**
* tracerouteコマンドの実装クラスです。
* ICMP echo requestを使用して、対象のアドレスまでのルーティング情報を
* 取得します。
*/
public class TraceRoute
{
	InetAddress	addr;
	Ping	ping;
	Vector<Route>	vecRoute = new Vector<Route>();
	int		times = 3;

	/**
	* 指定されたアドレスで、TraceRouteを構築します。
	* @param	addr	チェック対象のアドレス
	*/
	public TraceRoute( InetAddress addr ) throws Exception {
		this.addr = addr;
		ping = new Ping( addr );
	}

	/**
	* ルーティング情報の収集を開始します。
	*/
	public void check() throws Exception {
		int ttl = 0;
		for ( ;; ) {
			ttl++;
			if ( ttl > 64 ) {
				throw( new IllegalArgumentException( "ttl must < 64" ));
			}
			ping.setTTL( ttl );
			Route route = new Route();
			route.response = new long[times];
			vecRoute.add( route );
			for ( int i = 0; i < times; i++ ) {
				ping.check();
				Ping.Result res = ping.getResult( 0 );
				if ( res.isAlive()) {
					ICMPPacket recv = res.getReceivePacket();

					InetAddress src = InetAddress.getByAddress(
									recv.getHeader().getSource() );
					if ( route.addr == null ) {
						route.addr = src;
					}
					route.response[i] = res.response;
				} else {
					route.response[i] = -1;
				}
			}
			if ( addr.equals( route.addr )) {
				break;
			}
		}
	}

	/**
	* 収集したルーティング情報を返します。
	*/
	public List<Route> listRoutes() {
		return vecRoute;
	}

	/**
	* 個々のpingの結果を全て返します。
	*/
	public Ping.Result[] getResults() {
		return ping.getResults();
	}

	/**
	* ping結果を監視するIPingListenerを追加します。
	* @param	l		追加するListener
	*/
	public void addPingListener( IPingListener l ) {
		ping.addPingListener( l );
	}

	/**
	* ping結果を監視するIPingListenerを削除します。
	* @param	l		削除するListener
	*/
	public void removePingListener( IPingListener l ) {
		ping.removePingListener( l );
	}

	/**
	* 取得したルーティング情報を格納します。
	*/
	public static class Route
	{
		InetAddress	addr;
		long[]		response;
		/**
		* ルート上のアドレスを返します。
		*/
		public InetAddress getAddress() {
			return addr;
		}
		/**
		* アドレスのレスポンス時間を返します。
		*/
		public long[] getResponse() {
			return response;
		}
	}
}
